{
Ne winamp
---------
(c)2003 S.V.Bulba
http://bulba.at.kz/
vorobey@mail.khstu.ru

Description:
------------
Example of using lightBASS.pas and BASS.DLL sound library
- Loads BASS.DLL only before starting playing
- Automatically unloads BASS.DLL when Stop button pressed or
  playing of playlist comes to the end
- Can play MP3, MP2, MP1, OGG, WAV, MO3, IT, XM, S3M, MTM, MOD and UMX files
  from local drive and from intranet
- Supports time (for streams) and position visualisation and seeking
- Extracts Author and Title from ID3V1.x,ID3V2.3.x,ID3V2.4.x and OGG Vorbis
  tags, and from modules headers
}

unit MainWindow;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Buttons, StdCtrls;

type
  TFileTypes = (UnknownFile,MP3File,MP2File,MP1File,OGGFile,WAVFile,
   MO3File,ITFile,XMFile,S3MFile,MTMFile,MODFile,UMXFile);
  PID3v1 = ^TID3v1;
  TID3v1 = packed record
   Tag:array[0..2] of char;
   Title,Author,Album:array[0..29] of char;
   Year:array[0..3] of char;
   Comment:array[0..29] of char;
   Genre:byte;
  end;
  PID3V2Header = ^TID3V2Header;
  TID3V2Header = packed record
   Tag:array[0..2] of char;
   VerMajor,VerMinor,Flags:byte;
   Size:DWORD;
  end;
  PID3V2ExtHeader = ^TID3V2ExtHeader;
  TID3V2ExtHeader = packed record
   Size:DWORD;
   Flags:word;
   PaddingSize:DWORD;
  end;
  PID3V2Frame = ^TID3V2Frame;
  TID3V2Frame = packed record
   Id,Size:DWORD;
   Flags:word;
  end;
  TForm1 = class(TForm)
    Panel1: TPanel;
    PlayList: TListBox;
    AddButton: TSpeedButton;
    OpenDialog1: TOpenDialog;
    PlayButton: TSpeedButton;
    NextButton: TSpeedButton;
    PrevButton: TSpeedButton;
    StopButton: TSpeedButton;
    Bevel1: TBevel;
    ClearButton: TSpeedButton;
    PauseButton: TSpeedButton;
    ProgrBar: TScrollBar;
    CurTmLb: TLabel;
    TotTmLb: TLabel;
    Label3: TLabel;
    AboutButton: TSpeedButton;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure AddButtonClick(Sender: TObject);
    procedure PlayButtonClick(Sender: TObject);
    procedure NextButtonClick(Sender: TObject);
    procedure PrevButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure PauseButtonClick(Sender: TObject);
    procedure AboutButtonClick(Sender: TObject);
    procedure PlayCurrent;
    function PlayNext:boolean;
    procedure WM_NEXT(var Msg: TMessage);message WM_USER;
    procedure FreeAllBASS;
    procedure PlayListDblClick(Sender: TObject);
    procedure PlaySelected;
    procedure PlayListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure TimePosTimerProc(Sender: TObject);
    procedure StartTimePosTimer;
    procedure StopTimePosTimer;
    procedure ProgrBarScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    function GetSongTitle:string;
  private
    { Private declarations }
  public
    { Public declarations }
  end;

const
 MinFileType = Succ(Low(TFileTypes));
 MaxFileType = High(TFileTypes);
 StreamFileMin = MP3File;
 StreamFileMax = WAVFile;
 MpegFileMin = MP3File;
 MpegFileMax = MP1File;
 MODFilesMin = MO3File;
 MODFilesMax = UMXFile;
 FileTypes:array[MinFileType..MaxFileType] of string =
  ('.MP3','.MP2','.MP1','.OGG','.WAV','.MO3','.IT','.XM','.S3M','.MTM','.MOD',
   '.UMX');

var
  Form1: TForm1;
  CurrentItem:integer = -1;
  TimePosTimer:TTimer;
  TimerStarted:boolean = False;
  ScrollMoving:boolean;
  FileType:TFileTypes;

implementation

{$R *.DFM}

uses
 lightBASS;

function IntelDWord(DWrd:dword):dword;
asm
xchg al,ah
ror eax,16
xchg al,ah
end;
 
function TimeSToStr(t:real):string;
var
 ms:integer;
begin
ms := round(t);
SetLength(Result,4);
Result[4] := char(ms mod 10 + 48);
ms := ms div 10;
Result[3] := char(ms mod 6 + 48);
ms := ms div 6;
Result[2] := ':';
Result[1] := char(ms mod 10 + 48);
ms := ms div 10;
if ms = 0 then exit;
Result := char(ms mod 6 + 48) + Result;
ms := ms div 6;
if ms = 0 then exit;
Result := IntToStr(ms) + ':' + Result
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
FreeAllBASS
end;

procedure TForm1.AboutButtonClick(Sender: TObject);
begin
Application.MessageBox('Ne winamp'#13#10'Author is Sergey Bulba'#13#10 +
                       'Using BASS 1.8 by Ian Luck'#13#10 +
                       '(c)2003 S.V.Bulba'#13#10'http://bulba.at.kz/'#13#10 +
                       'vorobey@mail.khstu.ru','About program',MB_OK)
end;

procedure TForm1.AddButtonClick(Sender: TObject);
var
 i:integer;
begin
if OpenDialog1.Execute then
 begin
  OpenDialog1.InitialDir := ExtractFileDir(OpenDialog1.FileName);
  for i := 0 to OpenDialog1.Files.Count - 1 do
   PlayList.Items.Add(OpenDialog1.Files.Strings[i])
 end
end;

procedure TForm1.PlayButtonClick(Sender: TObject);
begin
PlayCurrent
end;

procedure TForm1.NextButtonClick(Sender: TObject);
begin
PlayNext
end;

procedure TForm1.PrevButtonClick(Sender: TObject);
begin
if CurrentItem > 0 then
 begin
  Dec(CurrentItem);
  PlayCurrent
 end
end;

procedure TForm1.StopButtonClick(Sender: TObject);
begin
FreeAllBASS
end;

procedure TForm1.ClearButtonClick(Sender: TObject);
begin
FreeAllBASS;
PlayList.Clear;
CurrentItem := -1
end;

procedure TForm1.PauseButtonClick(Sender: TObject);
begin
if MusicHandle = 0 then exit;
SwitchPause;
if not BASSPaused then
 PauseButton.Caption := 'Paus'
else
 PauseButton.Caption := 'Resu'
end;

function DetectFileType(FN:string):TFileTypes;
var
 i:TFileTypes;
begin
Result := UnknownFile;
FN := UpperCase(ExtractFileExt(FN));
for i := MinFileType to MaxFileType do
 if FileTypes[i] = FN then
  begin
   Result := i;
   exit
  end 
end;

procedure TForm1.PlayCurrent;
var
 timelen:DWORD;
begin
if PlayList.Items.Count = 0 then exit;
StopTimePosTimer;
PlayFree;
LoadBASS;
InitBASS(-1,44100,BASS_DEVICE_LEAVEVOL,Handle);
if CurrentItem = -1 then Inc(CurrentItem);
FileType := DetectFileType(PlayList.Items.Strings[CurrentItem]);
PlayBASS(PChar(PlayList.Items.Strings[CurrentItem]),
                FileType in [StreamFileMin..StreamFileMax]);
ProgrBar.Max := GetLengthBASS(timelen);
TotTmLb.Caption := TimeSToStr(BASS_ChannelBytes2Seconds(
                                        MusicHandle,timelen));
StartTimePosTimer;
PlayList.ItemIndex := CurrentItem;
Caption := GetSongTitle;
PauseButton.Caption := 'Paus'
end;

function TForm1.PlayNext;
begin
Result := CurrentItem < PlayList.Items.Count - 1;
if Result then
 begin
  Inc(CurrentItem);
  PlayCurrent
 end
end;

procedure TForm1.WM_NEXT;
begin
ProgrBar.Position := ProgrBar.Max;
CurTmLb.Caption := TotTmLb.Caption;
if not PlayNext then FreeAllBASS
end;

procedure TForm1.FreeAllBASS;
begin
StopTimePosTimer;
PlayFree;
FreeBASS;
UnloadBASS;
FileType := UnknownFile;
Caption := GetSongTitle
end;

procedure TForm1.PlayListDblClick(Sender: TObject);
begin
PlaySelected
end;

procedure TForm1.PlaySelected;
begin
if PlayList.ItemIndex >= 0 then
 begin
  CurrentItem := PlayList.ItemIndex;
  PlayCurrent
 end
end;

procedure TForm1.PlayListKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
if Key = VK_RETURN then PlaySelected
end;

procedure TForm1.TimePosTimerProc(Sender: TObject);
var
 i:DWORD;
begin
if MusicHandle = 0 then exit;
i := BASS_ChannelGetPosition(MusicHandle);
if MusicIsStream then
 CurTmLb.Caption := TimeSToStr(BASS_ChannelBytes2Seconds(MusicHandle,i))
else
 i := i and $FFFF;
if not ScrollMoving then ProgrBar.Position := i
end;

procedure TForm1.StartTimePosTimer;
begin
if TimerStarted then exit;
ScrollMoving := False;
ProgrBar.Position := 0;
CurTmLb.Caption := '0:00';
TimePosTimer := TTimer.Create(Self);
TimePosTimer.Interval := 100;
TimePosTimer.OnTimer := TimePosTimerProc;
TimerStarted := True
end;

procedure TForm1.StopTimePosTimer;
begin
if not TimerStarted then exit;
TimePosTimer.Free;
TimerStarted := False
end;

procedure TForm1.ProgrBarScroll(Sender: TObject; ScrollCode: TScrollCode;
  var ScrollPos: Integer);
begin
if MusicHandle = 0 then exit;
ScrollMoving := ScrollCode = scTrack;
if not BASSPaused and (ScrollCode = scEndScroll) then
 BASS_ChannelSetPosition(MusicHandle,ScrollPos)
end;

function RemoveStdExt(Ext:TFileTypes;FileName:string):string;
var
 SExt:string;
 i:integer;
begin
FileName := ExtractFileName(FileName);
SExt := Trim(FileName);
i := Length(SExt);
while (i > 1) and (SExt[i] <> '.') do Dec(i);
if i = 1 then
 begin
  Result := FileName;
  exit
 end;
SExt := UpperCase(Copy(SExt,i,Length(SExt) - i + 1));
if FileTypes[Ext] = SExt then
 Result := Copy(FileName,1,i - 1)
else
 Result := FileName
end;

function FormatTitle(Author,Title:string):string;
begin
if (Author <> '') and (Title <> '') then
 Result := Author + ' - ' + Title
else if Author <> '' then
 Result := Author
else if Title <> '' then
 Result := Title
else
 Result := ''
end;

function GetSongInfo_ID3V1:string;
var
 p:PID3v1;
 Author,Title:string;
begin
Result := '';
if MusicHandle = 0 then exit;
p := pointer(BASS_StreamGetTags(MusicHandle,BASS_TAG_ID3));
if p = nil then exit;
Author := p.Author;
Author := Trim(PChar(Author));
Title := p.Title;
Title := Trim(PChar(Title));
Result := FormatTitle(Author,Title)
end;

function GetSongInfo_ID3V2:string;

 function GetID3V2DWord(a:dword):dword;
 asm
  shl ah,1
  xchg al,ah
  shr ax,1
  ror eax,16
  shl ah,1
  xchg al,ah
  shl ax,1
  shr eax,2
 end;

const
 TIT2 = $32544954;
 TPE1 = $31455054;

var
 p,TagSize,AddByte,StrSize:integer;
 Author,Title:string;

 procedure CaseIds(Id:integer);
 begin
  case Id of
  TIT2:
   begin
    SetLength(Title,StrSize);
    Move(pointer(p + 10 + AddByte + 1)^,Title[1],StrSize);
    Title := Trim(PChar(Title))
   end;
  TPE1:
   begin
    SetLength(Author,StrSize);
    Move(pointer(p + 10 + AddByte + 1)^,Author[1],StrSize);
    Author := Trim(PChar(Author))
   end
  end
 end;

 procedure GetFromIDV24x; //not tested
 begin
  TagSize := DWORD(p) + GetID3V2DWord(PID3V2Header(p).Size);
  if PID3V2Header(p).Flags and 32 = 0 then
   Inc(TagSize,10);
  Inc(p,10);
  if PID3V2Header(p - 10).Flags and 64 <> 0 then
   Inc(p,GetID3V2DWord(PID3V2ExtHeader(p).Size));
  while p <= TagSize - SizeOf(TID3V2Frame) do
   with PID3V2Frame(p)^ do
    begin
     AddByte := Ord(Flags and 64 <> 0);
     StrSize := integer(GetID3V2DWord(Size)) - AddByte - 1;
     if (Flags and 15 = 0) and
        (StrSize > 0) and (StrSize <= TagSize - p - 10 - AddByte - 1) and
        (PByte(p + 10 + AddByte)^ = 0) then
      CaseIds(Id);
     Inc(p,GetID3V2DWord(Size) + 10)
    end
 end;

 procedure GetFromIDV23x;
 begin
  if PID3V2Header(p).Flags and 128 <> 0 then exit;
  TagSize := DWORD(p) + GetID3V2DWord(PID3V2Header(p).Size) + 10;
  Inc(p,10);
  if PID3V2Header(p - 10).Flags and 64 <> 0 then
   Inc(p,IntelDWord(PID3V2ExtHeader(p).Size));
  while p <= TagSize - SizeOf(TID3V2Frame) do
   with PID3V2Frame(p)^ do
    begin
     AddByte := Ord(Flags and 32 <> 0);
     StrSize := integer(IntelDWord(Size)) - AddByte - 1;
     if (Flags and (128 + 64) = 0) and (StrSize > 0) and
        (StrSize <= TagSize - p - 10 - AddByte - 1) and
        (PByte(p + 10 + AddByte)^ = 0) then
      CaseIds(Id);
     Inc(p,IntelDWord(Size) + 10)
    end
 end;

begin
Result := '';
if MusicHandle = 0 then exit;
p := integer(BASS_StreamGetTags(MusicHandle,BASS_TAG_ID3V2));
if p = 0 then exit;
Author := ''; Title := '';
case PID3V2Header(p).VerMajor of
4:GetFromIDV24x;
3:GetFromIDV23x
else
 exit
end;
Result := FormatTitle(Author,Title)
end;

function GetSongInfo_OGG:string;
var
 p:PChar;
 l,tl,cl:longword;
 Tag,Author,Title:string;
begin
Result := '';
if MusicHandle = 0 then exit;
p := BASS_StreamGetTags(MusicHandle,BASS_TAG_OGG);
if p = nil then exit;
Author := ''; Title := '';
repeat
 l := StrLen(p);
 tl := 0;
 while (tl < l) and (PByte(DWORD(p) + tl)^ <> Ord('=')) do Inc(tl);
 if (tl = l) or (tl = 0) then break;
 if tl < l - 1 then
  begin
   SetLength(Tag,tl);
   Move(p^,Tag[1],tl);
   cl := l - tl - 1;
   if UpperCase(Tag) = 'ARTIST' then
    begin
     SetLength(Author,cl);
     Move(pointer(DWORD(p) + tl + 1)^,Author[1],cl)
    end
   else if UpperCase(Tag) = 'TITLE' then
    begin
     SetLength(Title,cl);
     Move(pointer(DWORD(p) + tl + 1)^,Title[1],cl)
    end;
   SetLength(Tag,0)
  end;
 Inc(integer(p),l + 1)
until PByte(p)^ = 0;
Author := Trim(Author);
Title := Trim(Title);
Result := FormatTitle(Author,Title)
end;

function TForm1.GetSongTitle;
begin
if (FileType = UnknownFile) or (CurrentItem < 0) then
 Result := 'Ne winamp'
else
 begin
  Result := '';
  case FileType of
  MpegFileMin..MpegFileMax:
   begin
    Result := GetSongInfo_ID3V2;
    if Result = '' then
     Result := GetSongInfo_ID3V1
   end;
  OGGFile:
   Result := GetSongInfo_OGG;
  MODFilesMin..MODFilesMax:
   Result := Trim(BASS_MusicGetName(MusicHandle));
  end;
  if Result = '' then
   Result := RemoveStdExt(FileType,PlayList.Items.Strings[CurrentItem])
 end
end;

end.
